import numpy as np
from casadi import Function


def integrate_RK4(x_expr, u_expr, xdot_expr, dt, N_steps=1):

    h = dt/N_steps

    x_end = x_expr

    xdot_fun = Function('xdot', [x_expr, u_expr], [xdot_expr])

    for _ in range(N_steps):

        k_1 = xdot_fun(x_end, u_expr)
        k_2 = xdot_fun(x_end + 0.5 * h * k_1, u_expr) 
        k_3 = xdot_fun(x_end + 0.5 * h * k_2, u_expr)
        k_4 = xdot_fun(x_end + k_3 * h, u_expr)

        x_end = x_end + (1/6) * (k_1 + 2 * k_2 + 2 * k_3 + k_4) * h
    
    F_expr = x_end

    return F_expr

